@@ -7,5 +7,7 @@ from .task import Task, TaskStatus
 from .alert import Alert, AlertType
 from .source import Source, SourceFeature, FeatureType
 from .model import Model, ModelMethod, ModelType
-from .workspace import Workspace, Dashboard, Visualization
+from .workspace import Workspace
 from .organization import Organization
+from .visualization import Visualization
+from .dashboard import Dashboard
\ No newline at end of file
