@@ -3,143 +3,22 @@
 # Copyright 2021 Deep Intelligence
 # See LICENSE for details.
 
+import os
+import requests
 import pandas as pd
 from datetime import datetime
-from typing import Any, List, Dict, Optional, Generator
+from typing import Any, List, Dict, Optional, Generator, Union
+
+from .dashboard import Dashboard
+from .task import Task, TaskStatus
+from .alert import Alert, AlertType
+from .visualization import Visualization
+from .model import Model, ModelMethod, ModelType
+from .source import Source, SourceFeature, FeatureType
 
 from ..auth import Credentials
 from ..error import DeepintBaseError
 from ..util import handle_request, handle_paginated_request, parse_date, parse_url
-from .source import Source, SourceFeature
-from .model import Model, ModelMethod, ModelType
-from .task import Task, TaskStatus
-from .alert import Alert, AlertType
-
-
-class Visualization:
-    """Stores the information of a Deep Intelligence visualization.
-    
-    Attributes:
-        visualization_id: visualization's id in format uuid4.
-        name: visualization's name.
-        description: visualization's description.
-        created: creation date.
-        last_modified: last modified date.
-        last_access: last access date.
-        visualization_type: type of visualization.
-    """
-
-    def __init__(self, visualization_id: str, created: datetime, last_modified: datetime, last_access: datetime,
-                 name: str, description: str, visualization_type: str) -> None:
-        self.visualization_id = visualization_id
-        self.created = created
-        self.last_modified = last_modified
-        self.last_access = last_access
-        self.name = name
-        self.description = description
-        self.visualization_type = visualization_type
-
-    def __eq__(self, other):
-        if not isinstance(other, Visualization):
-            return False
-        else:
-            return self.visualization_id == other.visualization_id
-
-    def __str__(self):
-        return '<Visualization ' + ' '.join([f'{k}={v}' for k, v in self.to_dict().items()]) + '>'
-
-    @staticmethod
-    def from_dict(obj: Any) -> 'Visualization':
-        """Builds a Visualization with a dictionary.
-
-        Args:
-            obj: :obj:`object` or :obj:`dict` containing the a serialized Visualization.
-
-        Returns:
-            Visualization containing the information stored in the given dictionary.
-        """
-        visualization_id = obj.get("id")
-        created = parse_date(obj.get("created"))
-        last_modified = parse_date(obj.get("last_modified"))
-        last_access = parse_date(obj.get("last_access"))
-        name = obj.get("name")
-        description = obj.get("description")
-        visualization_type = obj.get("visualization_type")
-        return Visualization(visualization_id, created, last_modified, last_access, name, description,
-                             visualization_type)
-
-    def to_dict(self) -> Dict[str, Any]:
-        """Builds a dictionary containing the information stored in current object.
-
-        Returns:
-            dictionary containing the information stored in the current object.
-        """
-
-        return {"id": self.visualization_id, "created": self.created.isoformat(),
-                "last_modified": self.last_modified.isoformat(), "last_access": self.last_access.isoformat(),
-                "name": self.name, "description": self.description,
-                "visualization_type": self.visualization_type}
-
-
-class Dashboard:
-    """Stores the information of a Deep Intelligence dashboard.
-    
-    Attributes:
-        dashboard_id: dashboard's id in format uuid4.
-        name: dashboard's name.
-        description: dashboard's description.
-        created: creation date.
-        last_modified: last modified date.
-        last_access: last access date.
-    """
-
-    def __init__(self, dashboard_id: str, created: datetime, last_modified: datetime, last_access: datetime, name: str,
-                 description: str) -> None:
-        self.dashboard_id = dashboard_id
-        self.created = created
-        self.last_modified = last_modified
-        self.last_access = last_access
-        self.name = name
-        self.description = description
-
-    def __eq__(self, other):
-        if not isinstance(other, Dashboard):
-            return False
-        else:
-            return self.dashboard_id == other.dashboard_id
-
-    def __str__(self):
-        return '<Dashboard ' + ' '.join([f'{k}={v}' for k, v in self.to_dict().items()]) + '>'
-
-    @staticmethod
-    def from_dict(obj: Any) -> 'Dashboard':
-        """Builds a Dashboard with a dictionary.
-
-        Args:
-            obj: :obj:`object` or :obj:`dict` containing the a serialized Dashboard.
-
-        Returns:
-            Dashboard containing the information stored in the given dictionary.
-        """
-
-        dashboard_id = obj.get("id")
-        created = parse_date(obj.get("created"))
-        last_modified = parse_date(obj.get("last_modified"))
-        last_access = parse_date(obj.get("last_access"))
-        name = obj.get("name")
-        description = obj.get("description")
-        return Dashboard(dashboard_id, created, last_modified, last_access, name, description)
-
-    def to_dict(self) -> Dict[str, Any]:
-        """Builds a dictionary containing the information stored in current object.
-
-        Returns:
-            dictionary containing the information stored in the current object.
-        """
-
-        return {"id": self.dashboard_id, "created": self.created.isoformat(),
-                "last_modified": self.last_modified.isoformat(), "last_access": self.last_access.isoformat(),
-                "name": self.name, "description": self.description}
 
 
 class WorkspaceInfo:
@@ -237,7 +116,32 @@ class WorkspaceVisualizations:
         self._generator = None
         self._visualizations = visualizations
 
-    def load(self):
+    def create(self, name: str, description: str, privacy: str, source: str, configuration: Dict[str, Any] = {}) -> Visualization:
+        """Creates a visualization in current workspace.
+        
+        Args:
+            name: new visualization's name.
+            description: new visualization's description.
+            
+        Returns:
+            The created visualization
+        """
+        url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/visualizations'
+        parameters = {'name': name, 'description': description, 'privacy': privacy, 'source': source, 'configuration': configuration}
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_request(method='POST', url=url, headers=headers, parameters=parameters, credentials=self.workspace.credentials)
+
+        #map results
+        new_visualization = Visualization.build(workspace_id=self.workspace.info.workspace_id, visualization_id=response['visualization_id'],
+                            organization_id=self.workspace.organization_id, credentials=self.workspace.credentials)
+        
+        #update local state
+        self._visualizations = self._visualizations if self._visualizations is not None else []
+        self._visualizations.append(new_visualization)
+        
+        return new_visualization
+    
+    def load(self) -> None:
         """Loads a workspace's visualizations.
 
         If the visualizations were already loaded, this ones are replace by the new ones after retrieval.
@@ -245,11 +149,13 @@ class WorkspaceVisualizations:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/visualizations'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self._visualizations = None
-        self._generator = (Visualization.from_dict(v) for v in response)
+        self._generator = (Visualization.build(workspace_id=self.workspace.info.workspace_id, visualization_id=v['id'],
+                                                organization_id=self.workspace.organization_id, source_id=None, credentials=self.workspace.credentials) for v in response)
 
     def fetch(self, visualization_id: str = None, name: str = None, force_reload: bool = False) -> Optional[Visualization]:
         """Search for a visualization in the workspace.
@@ -332,6 +238,37 @@ class WorkspaceDashboards:
         self.workspace = workspace
         self._generator = None
         self._dashboards = dashboards
+    
+    def create(self, name: str, description: str, privacy: str, share_opt: str, restricted: bool, ga_id: str = None, configuration: Dict[str, Any] = {}) -> Dashboard:
+        """Creates a dashboard in the current workspace.
+        
+        Args
+            name: new dashboard's name
+            description: new dashboard's description
+            privacy: new dashboard's privacy
+            share_opt: Option for the shared dashboard GUI
+            ga_id: Opctional Google Analytics ID
+            restricted: True to check for explicit permission for shared dashboard
+            configuration: advanced option.
+        
+        Returns: 
+            The created dashboard
+        """
+
+        url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/dashboards'
+        parameters = {'name': name, 'description': description, 'privacy': privacy, 'shareOpt': share_opt, 'gaId': ga_id, 'restricted': restricted, 'configuration': configuration}
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_request(method='POST', url=url, parameters=parameters, headers=headers, credentials=self.workspace.credentials)
+        
+        # map result
+        new_dashboard = Dashboard.build(workspace_id=self.workspace.info.workspace_id, organization_id=self.workspace.organization_id,
+                                        dashboard_id=response['dashboard_id'], credentials=self.workspace.credentials)
+        
+        # update local state
+        self._dashboards = self._dashboards if self._dashboards is not None else []
+        self._dashboards.append(new_dashboard)
+
+        return new_dashboard
 
     def load(self):
         """Loads a workspace's dashboards.
@@ -341,12 +278,14 @@ class WorkspaceDashboards:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/dashboards'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self.dashboards = None
-        self._generator = (Dashboard.from_dict(d) for d in response)
-
+        self._generator = (Dashboard.build(organization_id=self.workspace.organization_id, workspace_id=self.workspace.info.workspace_id,
+                         dashboard_id=d['id'], credentials=self.workspace.credentials) for d in response)
+        
     def fetch(self, dashboard_id: str = None, name: str = None, force_reload: bool = False) -> Optional[Dashboard]:
         """Search for a dashboard in the workspace.
 
@@ -363,7 +302,7 @@ class WorkspaceDashboards:
                 :obj:`deepint.core.workspace.WorkspaceDashboards.load` method.
         
         Returns:
-            retrieved dashboard if found, and in other case None.
+            Retrieved dashboard if found, and in other case None.
         """
 
         # if set to true reload
@@ -393,7 +332,7 @@ class WorkspaceDashboards:
         The first time is invoked, builds a generator to retrieve dashboards directly from deepint.net API. However, 
         if there is stored dashboards and the force_reload option is not specified, only iterates in local 
         dashboards. In other case, it request the dashboards to deepint.net API and iterates over it.
-
+        
         Args:
             force_reload: if set to True, dashboards are reloaded before the search with the
                 :obj:`deepint.core.workspace.WorkspaceDashboard.load` method.
@@ -437,11 +376,13 @@ class WorkspaceSources:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/sources'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self._sources = None
         self._generator = (Source.build(workspace_id=self.workspace.info.workspace_id, source_id=s['id'],
+                                        organization_id=self.workspace.organization_id, 
                                         credentials=self.workspace.credentials) for s in response)
 
     def create(self, name: str, description: str, features: List[SourceFeature]) -> Source:
@@ -460,12 +401,13 @@ class WorkspaceSources:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/sources'
+        headers = {'x-deepint-organization': self.workspace.organization_id}
         parameters = {'name': name, 'description': description, 'features': [f.to_dict_minimized() for f in features]}
-        response = handle_request(method='POST', url=url, credentials=self.workspace.credentials, parameters=parameters)
+        response = handle_request(method='POST', url=url, headers=headers, credentials=self.workspace.credentials, parameters=parameters)
 
         # map results
         new_source = Source.build(source_id=response['source_id'], workspace_id=self.workspace.info.workspace_id,
-                                  credentials=self.workspace.credentials)
+                                  organization_id=self.workspace.organization_id, credentials=self.workspace.credentials)
 
         # update local state
         self._sources = self._sources if self._sources is not None else []
@@ -531,19 +473,22 @@ class WorkspaceSources:
         # if not exists, create
         return self.create(name, '', [])
 
-    def create_else_update(self, name:str, data: pd.DataFrame, **kwargs) -> Source:
+    def create_else_update(self, name:str, data: pd.DataFrame, delete_instances_on_feature_update: bool = True, **kwargs) -> Source:
         """Creates a source and initializes it, if it doesn't exist any source with same name. Else updates the source's instances. 
         
         The source is created with the :obj:`deepint.core.worksapce.WorkspaceSources.create_and_initialize`, so it's 
         reccomended to read the documentation of that method to learn more about the possible arguments of creation  (that can be 
-        providen in the **kwargs). Before creation, the source is loaded and stored locally in the internal list of sources in the 
+        providen in kwargs). Before creation, the source is loaded and stored locally in the internal list of sources in the 
         current instance. Also it's remmarkable that the source instance's are updated with the :obj:`deepint.core.source.SourceInstances.update`
         method, so it's reccomended to read the documentation of that method to learn more about the possible arguments of update (that 
-        can be providen in the **kwargs).
+        can be providen in the kwargs).
+
+        Note: if features change, then the source instances are deleted
 
         Args:
             name: source's name.
             data: data to in initialize the source. The source's feature names and data types are extracted from the given DataFrame. It the source also created, is updated with the given data.
+            delete_instances_on_feature_update: if set to False the instances are not deleted on features change.
 
         Returns:
             the affected source, updated if it was existing and created and initialized (if wait_for_initialization is providen and set to True) in other case.
@@ -554,7 +499,32 @@ class WorkspaceSources:
 
         # if exists update else create
         if selected_source is not None:
+            
+            # calculate features
+            new_features = SourceFeature.from_dataframe(df=data)
+
+            # update features if changed
+            for f in new_features:
+                current_feature = selected_source.features.fetch(name=f.name)
+                if (current_feature is None) or (f != current_feature):
+
+                    # delete previous instances
+                    if delete_instances_on_feature_update:
+                        selected_source.instances.clean()
+
+                    # update features
+                    t = selected_source.features.update(features=new_features)
+                    t.resolve()
+
+                    # update source
+                    selected_source.load()
+                    selected_source.features.load()
+
+                    break
+
+            # update source instances
             selected_source.instances.update(data=data,**kwargs)
+
         else:
             selected_source = self.create_and_initialize(name, '', data, **kwargs)
 
@@ -651,13 +621,14 @@ class WorkspaceTasks:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/tasks'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self._tasks = None
         self._generator = (
-            Task.build(workspace_id=self.workspace.info.workspace_id, credentials=self.workspace.credentials,
-                       task_id=t['id']) for t in response)
+            Task.build(organization_id=self.workspace.organization_id, workspace_id=self.workspace.info.workspace_id, 
+                       credentials=self.workspace.credentials, task_id=t['id']) for t in response)
 
     def fetch(self, task_id: str = None, name: str = None, force_reload: bool = False) -> Optional[Task]:
         """Search for a task in the workspace.
@@ -772,13 +743,14 @@ class WorkspaceAlerts:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/alerts'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self._alerts = None
         self._generator = (
-            Alert.build(workspace_id=self.workspace.info.workspace_id, credentials=self.workspace.credentials,
-                        alert_id=a['id']) for a in response)
+            Alert.build(organization_id=self.workspace.organization_id, workspace_id=self.workspace.info.workspace_id, 
+                        credentials=self.workspace.credentials, alert_id=a['id']) for a in response)
 
     def create(self, name: str, description: str, subscriptions: List[str], color: str, alert_type: AlertType,
                source_id: str, condition: dict = None, time_stall: int = None) -> Alert:
@@ -807,6 +779,7 @@ class WorkspaceAlerts:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/alerts'
+        headers = {'x-deepint-organization': self.workspace.organization_id}
         parameters = {
             'name': name,
             'description': description,
@@ -817,11 +790,11 @@ class WorkspaceAlerts:
             'condition': condition,
             'time_stall': time_stall
         }
-        response = handle_request(method='POST', url=url, credentials=self.workspace.credentials, parameters=parameters)
+        response = handle_request(method='POST', url=url, headers=headers, credentials=self.workspace.credentials, parameters=parameters)
 
         # map results
-        new_alert = Alert.build(workspace_id=self.workspace.info.workspace_id, credentials=self.workspace.credentials,
-                                alert_id=response['alert_id'])
+        new_alert = Alert.build(organization_id=self.workspace.organization_id, workspace_id=self.workspace.info.workspace_id, 
+                                credentials=self.workspace.credentials, alert_id=response['alert_id'])
 
         # update local state
         self._alerts = self._alerts if self._alerts is not None else []
@@ -919,13 +892,14 @@ class WorkspaceModels:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/models'
-        response = handle_paginated_request(method='GET', url=url, credentials=self.workspace.credentials)
+        headers = {'x-deepint-organization': self.workspace.organization_id}
+        response = handle_paginated_request(method='GET', url=url, headers=headers, credentials=self.workspace.credentials)
 
         # map results
         self._models = None
         self._generator = (
-            Model.build(workspace_id=self.workspace.info.workspace_id, credentials=self.workspace.credentials,
-                        model_id=m['id']) for m in response)
+            Model.build(organization_id=self.workspace.organization_id, workspace_id=self.workspace.info.workspace_id, 
+                        credentials=self.workspace.credentials, model_id=m['id']) for m in response)
 
     def create(self, name: str, description: str, model_type: ModelType, method: ModelMethod, source: Source,
                target_feature_name: str, configuration: dict = None, test_split_size: float = 0.3,
@@ -971,6 +945,7 @@ class WorkspaceModels:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.workspace.info.workspace_id}/models'
+        headers = {'x-deepint-organization': self.workspace.organization_id}
         parameters = {
             'name': name,
             'description': description,
@@ -986,17 +961,17 @@ class WorkspaceModels:
             },
             'hyper_search_configuration': hyper_parameters
         }
-        response = handle_request(method='POST', url=url, credentials=self.workspace.credentials, parameters=parameters)
+        response = handle_request(method='POST', url=url, headers=headers, credentials=self.workspace.credentials, parameters=parameters)
 
         # map response
         task = Task.build(task_id=response['task_id'], workspace_id=self.workspace.info.workspace_id,
-                          credentials=self.workspace.credentials)
+                          organization_id=self.workspace.organization_id, credentials=self.workspace.credentials)
 
         if wait_for_model_creation:
             # wait for task to finish and build model
             task.resolve()
             task_result = task.fetch_result()
-            new_model = Model.build(workspace_id=self.workspace.info.workspace_id,
+            new_model = Model.build(workspace_id=self.workspace.info.workspace_id, organization_id=self.workspace.organization_id, 
                                     credentials=self.workspace.credentials, model_id=task_result['model'])
 
             # update local state
@@ -1082,20 +1057,22 @@ class Workspace:
     or :obj:`deepint.core.workspace.Workspace.from_url` methods. 
     
     Attributes:
+        organization_id: the organziation where workspace is located.
         info: :obj:`deepint.core.workspace.WorkspaceInfo` to operate with workspace's information.
         tasks: :obj:`deepint.core.workspace.WorkspaceTasks` to operate with workspace's tasks.
         models: :obj:`deepint.core.workspace.WorkspaceModels` to operate with workspace's models.
         alerts: :obj:`deepint.core.workspace.WorkspaceAlerts` to operate with workspace's alerts.
-        sources: :obj:`deepint.core.workspace.WorkspaceDashboards` to operate with workspace's sources.
+        sources: :obj:`deepint.core.workspace.WorkspaceSources` to operate with workspace's sources.
         dashboards: :obj:`deepint.core.workspace.WorkspaceDashboards` to operate with workspace's dashboards.
         visualizations: :obj:`deepint.core.workspace.WorkspaceVisualizations` to operate with workspace's visualizations.
         credentials: credentials to authenticate with Deep Intelligence API and be allowed to perform operations over the workspace. If
                  not provided, the credentials are generated with the :obj:`deepint.auth.credentials.Credentials.build`.
     """
 
-    def __init__(self, credentials: Credentials, info: WorkspaceInfo, sources: List[Source], models: List[Model],
+    def __init__(self, organization_id: str, credentials: Credentials, info: WorkspaceInfo, sources: List[Source], models: List[Model],
                  tasks: List[Task], alerts: List[Alert], visualizations: List[Visualization],
                  dashboards: List[Dashboard]) -> None:
+        self.organization_id = organization_id
         self.info = info
         self.credentials = credentials
         self.tasks = WorkspaceTasks(self, tasks)
@@ -1112,13 +1089,14 @@ class Workspace:
             return self.info == other.info
 
     @classmethod
-    def build(cls, workspace_id: str, credentials: Credentials = None) -> 'Workspace':
+    def build(cls, organization_id: str, workspace_id: str, credentials: Credentials = None) -> 'Workspace':
         """Builds a workspace.
         
         Note: when workspace is created, the workspace's information and list of it's associated objects (tasks, models, sources, etc.) are loaded.
 
         Args:
-            workspace_id: workspace where workspace is located.
+            organization_id: organization where workspace is located.
+            workspace_id: workspace's id.
             credentials: credentials to authenticate with Deep Intelligence API and be allowed to perform operations over the workspace. If
                  not provided, the credentials are generated with the :obj:`deepint.auth.credentials.Credentials.build`.
 
@@ -1131,9 +1109,9 @@ class Workspace:
                              last_access=None, sources_count=None, dashboards_count=None, visualizations_count=None,
                              models_count=None,
                              size_bytes=None)
-        ws = cls(credentials=credentials, info=info, sources=None, models=None, tasks=None,
-                       alerts=None, visualizations=None, dashboards=None)
-
+        ws = cls(organization_id=organization_id, credentials=credentials, info=info, sources=None, models=None, 
+                       tasks=None, alerts=None, visualizations=None, dashboards=None)
+    
         ws.load()
         ws.tasks.load()
         ws.models.load()
@@ -1145,19 +1123,22 @@ class Workspace:
         return ws
 
     @classmethod
-    def from_url(cls, url: str, credentials: Credentials = None) -> 'Workspace':
+    def from_url(cls, url: str, organization_id: str = None, credentials: Credentials = None) -> 'Workspace':
         """Builds a workspace from it's API or web associated URL.
 
         The url must contain the workspace's id as in the following examples:
 
         Example:
-            - https://app.deepint.net/workspace?ws=f0e2095f-fe2b-479e-be4b-bbc77207f42d
+            - https://app.deepint.net/o/3a874c05-26d1-4b8c-894d-caf90e40078b/workspace?ws=f0e2095f-fe2b-479e-be4b-bbc77207f42d
             - https://app.deepint.net/api/v1/workspace/f0e2095f-fe2b-479e-be4b-bbc77207f42
         
         Note: when workspace is created, the workspace's information and list of it's associated objects (tasks, models, sources, etc.) are loaded.
+            Also it is remmarkable that if the API URL is providen, the organization_id must be provided in the optional parameter, otherwise
+            this ID won't be found on the URL and the Organization will not be created, raising a value error.
 
         Args:
             url: the workspace's API or web associated URL.
+            organization_id: the id of the organziation. Must be providen if the API URL is used.
             credentials: credentials to authenticate with Deep Intelligence API and be allowed to perform operations over the workspace. If
                  not provided, the credentials are generated with the :obj:`deepint.auth.credentials.Credentials.build`.
 
@@ -1167,10 +1148,14 @@ class Workspace:
 
         url_info = parse_url(url)
 
+        if 'organization_id' not in url_info and organization_id is None:
+            raise ValueError('Fields organization_id must be in url to build the object. Or providen as optional parameter.')
+
         if 'workspace_id' not in url_info:
             raise ValueError('Fields workspace_id must be in url to build the object.')
 
-        return cls.build(workspace_id=url_info['workspace_id'], credentials=credentials)
+        organization_id = url_info['organization_id'] if 'organization_id' in url_info else organization_id
+        return cls.build(organization_id=organization_id, workspace_id=url_info['workspace_id'], credentials=credentials)
 
     def load(self):
         """Loads the workspace's information.
@@ -1180,7 +1165,8 @@ class Workspace:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.info.workspace_id}'
-        response = handle_request(method='GET', url=url, credentials=self.credentials)
+        headers = {'x-deepint-organization': self.organization_id}
+        response = handle_request(method='GET', url=url, headers=headers, credentials=self.credentials)
 
         # map results
         self.info = WorkspaceInfo.from_dict(response)
@@ -1200,7 +1186,8 @@ class Workspace:
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.info.workspace_id}'
         parameters = {'name': name, 'description': description}
-        response = handle_request(method='POST', url=url, credentials=self.credentials, parameters=parameters)
+        headers = {'x-deepint-organization': self.organization_id}
+        response = handle_request(method='POST', url=url, headers=headers, parameters=parameters, credentials=self.credentials)
 
         # update local state
         self.info.name = name
@@ -1212,7 +1199,57 @@ class Workspace:
 
         # request
         url = f'https://app.deepint.net/api/v1/workspace/{self.info.workspace_id}'
-        handle_request(method='DELETE', url=url, credentials=self.credentials)
+        headers = {'x-deepint-organization': self.organization_id}
+        handle_request(method='DELETE', url=url, headers=headers, credentials=self.credentials)
+
+    def export(self, path: str=".", wait_for_download: bool = True, task: Task = None) -> Union[str, Task]:
+        """Exports a workspace to ZIP into the selected path.
+        
+        Args:
+            path: the path where the zip should be located. This parameter must contain the name of the file. By default is the
+                current folder.
+            wait_for_download: if set to true the file is located automatically into the selected path. In other case, the method
+                returns a :obj:`deepint.core.task.Task`, that can be used later to get the ZIP with this metod, providing it into
+                the task parameter.
+            task: :obj:`deepint.core.task.Task` used to obtain the URL to download the ZIP in a delayed download (when the wait_for_download parameter is set to false).
+        
+        Returns:
+            The path to downloaded ZIP in the case of wait_for_download is set to True. In other case the task generated to build the ZIP.
+        """
+
+        if task is None:
+
+            # build request
+            url = f'https://app.deepint.net/api/v1/workspace/{self.info.workspace_id}/export'
+            headers = {'x-deepint-organization': self.organization_id}
+            response = handle_request(method='POST', url=url, headers=headers,credentials=self.credentials)
+
+            # create task to fetch the ZIP file
+            task = Task.build(task_id=response['task_id'], workspace_id=self.info.workspace_id,
+                              organization_id=self.organization_id, credentials=self.credentials)
+
+            if not wait_for_download:
+                return task
+
+        # wait for task resolution to obtain reslult
+        task.resolve()
+        result = task.fetch_result()
+
+        if 'file' not in result:
+            raise DeepintBaseError(code='DOWNLOAD_FAILED', message="The task generated to build the ZIP file failed, please try again in a few seconds")
+
+        file_url = result['file']
+
+        # download and store ZIP file
+        try:            
+            file_path = os.path.join(path, f'{self.info.workspace_id}.zip')
+            file_path = os.path.abspath(file_path)
+            r = requests.get(file_url)
+            open(file_path, 'wb').write(r.content)
+            return file_path
+        except Exception as e:
+            raise e
+            raise DeepintBaseError(code='DOWNLOAD_FAILED', message="Unable to write the file's content. Check the path and permisions.")
 
     def to_dict(self) -> Dict[str, Any]:
         """Builds a dictionary containing the information stored in current object.
